Object.defineProperty(exports, "__esModule", { value: true });
class PriorityQueue {
    constructor(comparator) {
        this._queue = [];
        this._comparator = comparator;
    }
    push(value) {
        this._queue.push(value);
        let pos = this._queue.length - 1;
        while (pos !== 0 &&
            this._compare(this._queue[this._parentOf(pos)], this._queue[pos])) {
            this._swap(pos, this._parentOf(pos));
            pos = this._parentOf(pos);
        }
    }
    top() {
        return this._queue.length > 0 ? this._queue[0] : null;
    }
    pop() {
        if (this._queue.length === 0) {
            return null;
        }
        const item = this._queue[0];
        this._queue[0] = this._queue[this._queue.length - 1];
        this._swap(0, this._queue.length - 1);
        this._queue.pop();
        this._heapify(0);
        return item;
    }
    size() {
        return this._queue.length;
    }
    empty() {
        return this._queue.length === 0;
    }
    toArray() {
        return [...this._queue];
    }
    _heapify(index) {
        const mid = Math.floor(this._queue.length / 2);
        let childIndex1;
        let childIndex2;
        let swapIndex;
        while (index <= mid - 1) {
            childIndex1 = 2 * index + 1;
            childIndex2 = 2 * index + 2;
            swapIndex = childIndex1;
            if (this._compare(this._queue[childIndex1], this._queue[childIndex2])) {
                swapIndex = childIndex2;
            }
            if (this._compare(this._queue[index], this._queue[swapIndex])) {
                this._swap(index, swapIndex);
            }
            index = swapIndex;
        }
    }
    _swap(index1, index2) {
        const temp = this._queue[index1];
        this._queue[index1] = this._queue[index2];
        this._queue[index2] = temp;
    }
    _compare(item1, item2) {
        if (this._comparator) {
            return this._comparator(item1, item2);
        }
        return item1 < item2;
    }
    _parentOf(position) {
        return Math.floor((position - 1) / 2);
    }
}
exports.PriorityQueue = PriorityQueue;
